/******************************************************************************
*
* project name:
* file name:       tilegen.c
* initial date:    27/06/2001
* authors:         thomas.nussbaumer@gmx.net
* description:     generates a tile file
*
* $Id$
*
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define MAGIC_MARKER

//=============================================================================
// outputs usage information of this tool
//=============================================================================
void PrintUsage() {
    fprintf(stdout, "Usage: tilegen <tiles_per_row> <infile> <outfile>\n"\
                    "       tiles_per_row ... number of horizontal arranged tiles in infile\n\n"\
                    "       generates a binary file of tiles of size 16x16 out of an\n"\
                    "       ImageStudio grayscale export file.\n\n"\
                    "NOTE:  to overcome the restriction of ImageStudio which converts only\n"\
                    "       to a maximum size of 240x128 pixels just append multiple\n"\
                    "       binary ImageStudio files of the same width to each other\n\n");
}

//-----------------------------------------------------------------------------
// globals
//
// to reduce the function parameters we are using almost only globals
// (bad idea in general, but perfect for rapid prototyping)
//-----------------------------------------------------------------------------
FILE*          outfp;
int            hcount;
int            vcount;
long           insize;
unsigned char* buffer;



//=============================================================================
// extract complete tile
//=============================================================================
void WriteTile(long offset_plane2,int x,int y) {
    int h;
    int bytes_per_line = hcount*16/8;

    unsigned char* addr = buffer + y*bytes_per_line + x/8;
    unsigned int  data1;
    unsigned int  shift = x % 8;

    for (h=0;h<16;h++) {
        data1 =  (*addr & 0xff) << 16;
        data1 |= (*(addr+1) & 0xff) << 8;
        data1 |= (*(addr+2) & 0xff);
        data1 = (data1 >> (8-shift));
        fputc((data1>>8) & 0xff,outfp);
        fputc(data1 & 0xff,outfp);
        addr += bytes_per_line;
    }

    addr = buffer + offset_plane2 + y*bytes_per_line + x/8;
    shift = x % 8;

    for (h=0;h<16;h++) {
        data1 =  (*addr & 0xff) << 16;
        data1 |= (*(addr+1) & 0xff) << 8;
        data1 |= (*(addr+2) & 0xff);
        data1 = (data1 >> (8-shift));
        fputc((data1>>8) & 0xff,outfp);
        fputc(data1 & 0xff,outfp);
        addr += bytes_per_line;
    }

}


//=============================================================================
// process a complete plane starting at given offset
//=============================================================================
void ProcessTiles(long offset_plane2) {
    int x;
    int y;

    for (y=0;y<vcount;y++) {
        for (x=0;x<hcount;x++) {
            WriteTile(offset_plane2,x*16,y*16);
        }
    }
}

//=============================================================================
// nomen omen est
//=============================================================================
int main(int argc, char* argv[]) {
    FILE* infp;

    if (argc != 4) {
        PrintUsage();
        return 1;
    }

    if (sscanf(argv[1],"%d",&hcount) != 1) {
        fprintf(stderr,"ERROR: cannot parse width=%s\n",argv[3]);
        PrintUsage();
        return 1;
    }


    //vcount = height / tileheight;

    //-------------------------------------------------------------------------
    // open inputfile
    //-------------------------------------------------------------------------
    if (!(infp = fopen(argv[2],"rb"))) {
        fprintf(stderr,"ERROR: cannot open inputfile %s\n",argv[2]);
        PrintUsage();
        return 1;
    }

    //-------------------------------------------------------------------------
    // get and check input size
    //-------------------------------------------------------------------------
    if (fseek(infp,0,SEEK_END)) {
        fprintf(stderr,"ERROR: cannot find end of inputfile %s\n",argv[2]);
        PrintUsage();
        fclose(infp);
        return 1;
    }

    insize = ftell(infp);
    if (insize == -1L) {
        fprintf(stderr,"ERROR: cannot evaluate size of inputfile %s\n",argv[2]);
        PrintUsage();
        fclose(infp);
        return 1;
    }
    rewind(infp);

    vcount = (insize/64)/hcount;

    if (hcount*vcount*64 != insize) {
        fprintf(stderr,"ERROR: filesize (%ld) doesn't correspond with width and 16x16 grayscale tiles\n",insize);
        PrintUsage();
        fclose(infp);
        return 1;
    }

    //-------------------------------------------------------------------------
    // allocate buffer, read in all bytes and close input file
    //-------------------------------------------------------------------------
    if (!(buffer = malloc(insize))) {
        fprintf(stderr,"ERROR: cannot allocate %ld bytes for input data\n",insize);
        PrintUsage();
        fclose(infp);
        return 1;
    }

    if (fread(buffer,insize,1,infp) != 1) {
        fprintf(stderr,"ERROR: cannot read %ld bytes from inputfile %s\n",insize,argv[2]);
        PrintUsage();
        fclose(infp);
        free(buffer);
        return 1;
    }

    fclose(infp);

    //-------------------------------------------------------------------------
    // open outputfile
    //-------------------------------------------------------------------------
    if (!(outfp = fopen(argv[3],"wb"))) {
        fprintf(stderr,"ERROR: cannot open outputfile %s\n",argv[3]);
        PrintUsage();
        free(buffer);
        return 1;
    }

    fputc(0x54,outfp);
    fputc(0x69,outfp);
    fputc(0x4C,outfp);
    fputc(0x65,outfp);
    fputc(((vcount*hcount)>>8) & 0xff,outfp);
    fputc((vcount*hcount) & 0xff,outfp);
    ProcessTiles(insize/2);

    fclose(outfp);
    free(buffer);

    return 0;
}


//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log$
//
