/******************************************************************************
*
* project name:  TICT Game Construction Kit (TICT-GCK)
* file name:     gcklib.h
* initial date:  27/06/2001
* author:        thomas.nussbaumer@gmx.net
* description:   headerfile of TICT Game Construction Kit
*
* $Id$
*
******************************************************************************/

#ifndef __GCKLIB_H__
#define __GCKLIB_H__

#include <vat.h>      // for type SYM_ENTRY
#include <compat.h>   // for macro TI89
#include <graph.h>    // for function PortSet

#define GCKLIB_VERSION   "GCKLIB v0.35"
#define C89_92(a,b)      ((TI89) ? (a) : (b))

typedef unsigned short   KBSTATE;

//-----------------------------------------------------------------------------
// WARNING: DON'T USE ANY OF THE FOLLOWING VARIABLES DIRECTLY, BUT USE THE
//          CORRESPONDING MACROS AND FUNCTIONS INSTEAD !!!
//
// (the names of these variables may vary in the next library version or the
//  variables may become completely removed)
//-----------------------------------------------------------------------------
extern unsigned char*         __backbuffer1__;   // light backbuffer plane addr.
extern unsigned char*         __backbuffer2__;   // dark  backbuffer plane addr.
extern short                  __displayoffset__; // display offset in bytes used for TI92p
extern volatile unsigned long __time__base__;    // the timecounter incremented by INT5 handler
extern short                  __int1__state__;   // state of default INT1 handler execution
extern short                  __int5__state__;   // state of default INT5 handler execution
extern unsigned short*        __map_pointer__;   // pointer to active map
extern unsigned short*        __tiles_pointer__; // pointer to active tile data
extern short                  __map_size_x__;    // number of columns of active map
extern short                  __map_size_y__;    // number of rows of active map
extern short                  __map_pos_x__;     // active map x position (upper/left corner)
extern short                  __map_pos_y__;     // active map y position (upper/left corner)
extern short                  __map_pos_x_max__; // max. map x position for upper/left corner
extern short                  __map_pos_y_max__; // max. map y position for upper/left corner
extern  KBSTATE              (*__readkeys__)(void);   // pointer to used readkey function
extern void                  (*__sleep__ms__)(unsigned short); // pointer to used sleep_ms function



// ============================================================================
// ============================================================================
// ==  bounding box collision detection                                      ==
// ============================================================================
// ============================================================================

// Bounding Box collision detection (16x16 pixel sized-rectangles)
short TestBoundCollide16x16(short x0,short y0,short x1, short y1);



// ============================================================================
// ============================================================================
// ==  keyboard handling                                                     ==
// ============================================================================
// ============================================================================

//-----------------------------------------------------------------------------
// the keyboard reading routine mimic the keyboard routine of the genlib in
// mean of which keys are used on which calculator.
//
// In detail the following keys are queried (depending on the calc type):
//
// flag set           TI-89     TI-92
// -------------------------------------------
// KEYFLAGS_ESC       ESC       ESC
// KEYFLAGS_UP        up        up
// KEYFLAGS_DOWN      down      down
// KEYFLAGS_LEFT      left      left
// KEYFLAGS_RIGHT     right     right
// KEYFLAGS_MODE      APPS      APPS
// KEYFLAGS_PLUS      +         +
// KEYFLAGS_MINUS     -         -
// KEYFLAGS_KA        2nd       F1
// KEYFLAGS_KB        Diamond   F2
// KEYFLAGS_KC        Home      F3
// KEYFLAGS_KD        X         F4
// KEYFLAGS_KE        Shift     F5
// KEYFLAGS_KF        Alpha     F6
// KEYFLAGS_KG        Mode      F7
// KEYFLAGS_KH        Y         F8
//
//-----------------------------------------------------------------------------

#define KEYFLAGS_ESC     0x0001
#define KEYFLAGS_UP      0x0002
#define KEYFLAGS_DOWN    0x0004
#define KEYFLAGS_LEFT    0x0008
#define KEYFLAGS_RIGHT   0x0010
#define KEYFLAGS_MODE    0x0020
#define KEYFLAGS_PLUS    0x0040
#define KEYFLAGS_MINUS   0x0080
#define KEYFLAGS_KA      0x0100
#define KEYFLAGS_KB      0x0200
#define KEYFLAGS_KC      0x0400
#define KEYFLAGS_KD      0x0800
#define KEYFLAGS_KE      0x1000
#define KEYFLAGS_KF      0x2000
#define KEYFLAGS_KG      0x4000
#define KEYFLAGS_KH      0x8000

#define GetKeyboardState __readkeys__
void    WaitForAllKeysReleased(void);



// ============================================================================
// ============================================================================
// ==  initialization and cleanup                                            ==
// ============================================================================
// ============================================================================

//-----------------------------------------------------------------------------
// initializes GCK library
//
// (1) initialize backbuffer (allocates it) - returns immediately 0 if failed
// (2) installs special INT1 and INT5 handlers to provide calculator type
//     independent lowlevel keyboard reading
// (3) initializes timecounter support
// (4) initializes HW version independent SleepForMilliseconds routine
// (5) Turns grayscales on
//
// NOTE: if your program wants to install its own interrupt handlers for
//       INT1 and/or INT5 you have to install them BEFORE you call
//       calling InitGCKLibrary()
//-----------------------------------------------------------------------------
short InitGCKLibrary(void);

//-----------------------------------------------------------------------------
// cleanup of GCK library
//
// (1) turns grayscales off
// (2) frees backbuffer
// (3) restores default INT1 and INT5 handlers
//
// NOTE: if your program uses own interrupt handlers for INT1 and/or INT5
//       you have to remove them AFTER calling CleanupGCKLibrary()
//-----------------------------------------------------------------------------
void CleanupGCKLibrary(void);



// ============================================================================
// ============================================================================
// ==  time related routines                                                 ==
// ============================================================================
// ============================================================================

//-----------------------------------------------------------------------------
// 1 tick == approx. 50 ms)
//-----------------------------------------------------------------------------
#define GetActualTicks()       __time__base__
#define SetActualTicks(val)    ({__time__base__ = val;})
#define SleepForMilliseconds   __sleep__ms__
void    SleepForTicks(unsigned long nr_ticks);



// ============================================================================
// ============================================================================
// ==  interrupt handler related stuff                                       ==
// ============================================================================
// ============================================================================
#define DEFAULT_INT1_EXEC_STATE         1
#define DEFAULT_INT5_EXEC_STATE         1
#define SetDefaultINT1Exec(boolean_val) ({__int1__state__ = boolean_val;})
#define SetDefaultINT5Exec(boolean_val) ({__int5__state__ = boolean_val;})
#define GetDefaultINT1Exec()            __int1__state__
#define GetDefaultINT5Exec()            __int5__state__



// ============================================================================
// ============================================================================
// ==  misc. routines and macros                                             ==
// ============================================================================
// ============================================================================
void          DrawBorder92(void);
unsigned long GetHardwareVersion(void);
#define       GetDisplayOffsetInBytes() __displayoffset__



// ============================================================================
// ============================================================================
// ==  load / save related stuff                                             ==
// ============================================================================
// ============================================================================
#define MAGIC_TILEFILE 0x54694C65  // "TiLe"
#define IS_TILEFILE(ptr) (*((unsigned long*)(((unsigned char*)(ptr))+2)) == MAGIC_TILEFILE)

#define MAGIC_MAPFILE  0x4D615066  // "MaPf"
#define IS_MAPFILE(ptr) (*((unsigned long*)(((unsigned char*)(ptr))+2)) == MAGIC_MAPFILE)

typedef struct {
    unsigned short nr_tiles;
    unsigned short data[0];
} TILEDATA;

typedef struct {
    unsigned short  size_x;
    unsigned short  size_y;
    char            tilefile[10]; // normally 9, but 10 for alignment
    unsigned short  data[0];
} MAPDATA;

char*       GenerateTIOSName(char* cname,char* buffer);
SYM_ENTRY*  GetSymByName(char* name);
TILEDATA*   LoadTileData(char* filename);
MAPDATA*    LoadMapData(char* filename);
short       SaveMapData(char* filename,MAPDATA* md);
SYM_ENTRY*  FileSelection(char* dialogtitle,short(*callback)(unsigned char*),short *found);



// ============================================================================
// ============================================================================
// ==  sprite and tile drawing routines                                      ==
// ============================================================================
// ============================================================================
void PutSprite16x16(short x,short y,unsigned short* spritedata);
void PutSprite16x16Clipped(short x,short y,unsigned short* spritedata);
void PutTile16x16(short x,short y,unsigned short* tiledata);
void PutTile16x16Clipped(short x,short y,unsigned short* tiledata);



// ============================================================================
// ============================================================================
// ==  map and backbuffer related routines                                   ==
// ============================================================================
// ============================================================================
#define GetBackbuffer(plane)      ((!(plane)) ? __backbuffer1__ : __backbuffer2__)
#define BackbufferPortSet(plane)  PortSet(GetBackbuffer(plane),239,127)

#define MASK_TILENR               0x03FF   // used to mask out the tile number

void           SetMap(MAPDATA* mapfile,TILEDATA* tiledata,short pos_x,short pos_y);
short          SetMapPosX(short x);
short          SetMapPosY(short y);
void           DrawMap(void);
void           CopyBackbuffer2Screen(void);
unsigned short GetMapContentAtScreenXY(short screen_x,short screen_y);

#define        GetMapPosX()                     __map_pos_x__
#define        GetMapPosY()                     __map_pos_y__
#define        GetMapPosX_MAX()                 __map_pos_x_max__
#define        GetMapPosY_MAX()                 __map_pos_y_max__
#define        ModifyMapPosX(delta_x)           SetMapPosX(__map_pos_x__ + (delta_x))
#define        ModifyMapPosY(delta_y)           SetMapPosY(__map_pos_y__ + (delta_y))
#define        ScreenX2MapX(screen_x)           ((__map_pos_x__+(screen_x))>>4)
#define        ScreenX2TileX(screen_x)          ((__map_pos_x__+(screen_x))%16)
#define        ScreenY2MapY(screen_y)           ((__map_pos_y__+(screen_y))>>4)
#define        ScreenY2TileY(screen_y)          ((__map_pos_y__+(screen_y))%16)
#define        GetMapContent(map_x,map_y)       (*(__map_pointer__+(map_y)*__map_size_x__+(map_x)))
#define        SetMapContent(map_x,map_y,value) ({*(__map_pointer__+(map_y)*__map_size_x__+(map_x)) = value;})



// ============================================================================
// ============================================================================
// ==  unpack stuff from TTools Suite (just renamed to prevent conflicts)    ==
// ============================================================================
// ============================================================================
typedef struct {
    unsigned char  osize_lo;   // original size lowbyte
    unsigned char  osize_hi;   // original size highbyte
    unsigned char  magic1;     // must be equal to TTUNPACK_MAGIC1
    unsigned char  magic2;     // must be equal to TTUNPACK_MAGIC2
    unsigned char  csize_lo;   // compressed size lowbyte
    unsigned char  csize_hi;   // compressed size lowbyte
    unsigned char  esc1;       // escape >> (8-escBits)
    unsigned char  notused3;
    unsigned char  notused4;
    unsigned char  esc2;       // escBits
    unsigned char  gamma1;     // maxGamma + 1
    unsigned char  gamma2;     // (1<<maxGamma)
    unsigned char  extralz;    // extraLZPosBits
    unsigned char  notused1;
    unsigned char  notused2;
    unsigned char  rleentries; // rleUsed
} __TTUNPACK_HEADER__;

#define __TTUNPACK_MAGIC1__ 0x54
#define __TTUNPACK_MAGIC2__ 0x50

#define GetUnpackedSize(_p_)  ((unsigned short)(((__TTUNPACK_HEADER__*)(_p_))->osize_lo | (((__TTUNPACK_HEADER__*)(_p_))->osize_hi << 8)))
#define IsValidPacked(_p_)    (((__TTUNPACK_HEADER__*)(_p_))->magic1 == __TTUNPACK_MAGIC1__ && ((__TTUNPACK_HEADER__*)(_p_))->magic2 == __TTUNPACK_MAGIC2__)

#define UNPACK_OKAY             0
#define UNPACK_NOESCFOUND     248
#define UNPACK_ESCBITS        249
#define UNPACK_MAXGAMMA       250
#define UNPACK_EXTRALZP       251
#define UNPACK_NOMAGIC        252
#define UNPACK_OUTBUFOVERRUN  253
#define UNPACK_LZPOSUNDERRUN  254

extern unsigned short __unpack__(unsigned char* src,unsigned char* dest);

#define Unpack __unpack__

/*
#define Unpack ((unsigned short(*)(unsigned char*,unsigned char*))(unsigned short[]){\
0x48e7,0x7ffa,0x4fef,0xffe8,0x206f,0x0050,0x0c28,0x0054,0x0002,0x6608,\
0x0c28,0x0050,0x0003,0x6708,0x303c,0x00fc,0x6000,0x0264,0x4286,0x1c28,\
0x0006,0x4247,0x1e28,0x0009,0x4245,0x1a28,0x000a,0x5345,0x4240,0x1028,\
0x000c,0x3c40,0x7201,0xeb69,0x3f41,0x000e,0x7002,0xeb68,0x3f40,0x000c,\
0x7208,0x9245,0x3f41,0x000a,0x7008,0x9047,0x3840,0x303c,0x00f9,0x0c47,\
0x0008,0x6200,0x021e,0x4240,0x1028,0x000b,0xb06f,0x000e,0x660c,0x0c45,\
0x0004,0x6306,0x0c45,0x0007,0x6308,0x303c,0x00fa,0x6000,0x01fe,0x7204,\
0xb24e,0x6408,0x303c,0x00fb,0x6000,0x01f0,0x43e8,0x000f,0x2f49,0x0010,\
0x266f,0x0054,0x43fa,0x01ea,0x2f49,0x0006,0x226f,0x0010,0x4280,0x1011,\
0x41f0,0x0810,0x226f,0x0006,0x2288,0x41fa,0x01d4,0x2f48,0x0002,0x30bc,\
0x0080,0x3606,0x4a47,0x670c,0x3f07,0x41fa,0x0274,0x4e90,0x3600,0x548f,\
0x3043,0xbc88,0x6600,0x018c,0x3f05,0x45fa,0x021a,0x4e92,0x3800,0x426f,\
0x0002,0x548f,0x0c44,0x0001,0x6754,0x3f05,0x4e92,0x3600,0x5343,0x302f,\
0x000e,0x5540,0x548f,0xb043,0x6618,0x0c44,0x0003,0x6300,0x0170,0x45fa,\
0x017e,0x4e92,0x3f40,0x0000,0x4e92,0x6000,0x0120,0x300e,0x6710,0x3f0e,\
0x41fa,0x021a,0x4e90,0x320e,0xe36b,0x8640,0x548f,0x41fa,0x015a,0x4e90,\
0x0a40,0x00ff,0x3203,0xe149,0x8240,0x6000,0x00fe,0x226f,0x0006,0x2051,\
0x4240,0x1010,0x206f,0x0002,0x3210,0xc041,0x6700,0x00ca,0xe219,0x6402,\
0x5291,0x3081,0x2051,0x4240,0x1010,0xc041,0x6630,0x3001,0xe218,0x6402,\
0x5291,0x226f,0x0002,0x3280,0x3f07,0x45fa,0x01c0,0x4e92,0x3600,0x3f0c,\
0x4e92,0x320c,0x48c1,0xe3ae,0x8c00,0x16c6,0x4286,0x3c03,0x588f,0x6000,\
0xff26,0x3001,0xe218,0x6402,0x5291,0x206f,0x0002,0x3080,0x3f05,0x4e92,\
0x3600,0x548f,0xb66f,0x000e,0x6522,0x3f2f,0x000a,0x41fa,0x0180,0x4e90,\
0x966f,0x0010,0x322f,0x000c,0xe36b,0x8640,0x3f05,0x4e92,0x5340,0xe148,\
0x8640,0x588f,0x3f05,0x4e92,0x3800,0x548f,0x0c44,0x001f,0x6210,0x4280,\
0x3004,0x206f,0x0010,0x1230,0x0800,0x4881,0x6016,0x3f3c,0x0003,0x41fa,\
0x0140,0x4e90,0x3204,0x0641,0xffe0,0xe749,0x8240,0x548f,0x4242,0xb642,\
0x6500,0xfeac,0x16c1,0x5242,0xb642,0x64f8,0x6000,0xfea0,0x3001,0x226f,\
0x0006,0xe218,0x6402,0x5291,0x206f,0x0002,0x3080,0x41fa,0x0056,0x4e90,\
0x3200,0x0a41,0x00ff,0x4242,0xb842,0x6500,0xfe7a,0x4280,0x3001,0x204b,\
0x91c0,0x102f,0x0001,0xd028,0xffff,0x16c0,0x5242,0xb842,0x64e8,0x6000,\
0xfe5e,0x3f0c,0x41fa,0x00d6,0x4e90,0x320c,0xe36b,0x8600,0x16c3,0x548f,\
0x6000,0xfe48,0x4240,0x4fef,0x0018,0x4cdf,0x5ffe,0x4e75,0x0000,0x0000,\
0x0000,0x41fa,0xfff8,0x2250,0x4240,0x1019,0x2089,0x323a,0xfff0,0x0c01,\
0x0080,0x6602,0x4e75,0xe148,0x1011,0x0c01,0x0008,0x6212,0x6630,0x0240,\
0x0fff,0xe848,0x4e75,0x0240,0x7fff,0xee48,0x4e75,0x0c01,0x0020,0x62f2,\
0x6508,0x0240,0x3fff,0xec48,0x4e75,0x0240,0x1fff,0xea48,0x4e75,0x0240,\
0x07ff,0xe648,0x4e75,0x0c01,0x0002,0x62f2,0x6608,0x0240,0x03ff,0xe448,\
0x4e75,0x0240,0x01ff,0xe248,0x4e75,0x2f03,0x4281,0x302f,0x0008,0x207a,\
0xff84,0x343a,0xff84,0xb041,0x6312,0x3602,0xc610,0xe21a,0x6402,0x5288,\
0x4a03,0x6704,0x5241,0x60ea,0x2608,0x41fa,0xff64,0x2083,0x41fa,0xff62,\
0x3082,0x2043,0x7601,0xe36b,0x3f01,0x6108,0x544f,0x8043,0x261f,0x4e75,\
0x4240,0x322f,0x0004,0x6730,0x207a,0xff3e,0x2243,0x343a,0xff3c,0x5341,\
0xe348,0x3602,0xc610,0x6702,0x5200,0xe21a,0x6402,0x5288,0x51c9,0xffee,\
0x2609,0x43fa,0xff20,0x3282,0x43fa,0xff16,0x2288,0x2243,0x4e75})
*/
#endif

//#############################################################################
// Revision History
//#############################################################################
//
// $Log$
//
